searchData={"items":[{"type":"module","title":"Talan","doc":"Fast & concurrent probabilistic data structures\nbuilt on top of :atomics with **concurrent accessibility**.\n\n  * `Talan.BloomFilter` - bloom filter based on `:atomics`\n  * `Talan.CountingBloomFilter` - counting bloom filter based on `:atomics`\n  * `Talan.Counter` - linear probabilistic counter based on `:atomics`","ref":"Talan.html"},{"type":"module","title":"Talan.BloomFilter","doc":"Bloom filter implementation with **concurrent accessibility**,\npowered by [:atomics](http://erlang.org/doc/man/atomics.html) module.\n\n\"A Bloom filter is a space-efficient probabilistic data structure,\nconceived by Burton Howard Bloom in 1970,\nthat is used to test whether an element is a member of a set\"\n\n[Bloom filter on Wikipedia](https://en.wikipedia.org/wiki/Bloom_filter#CITEREFZhiwangJungangJian2010)","ref":"Talan.BloomFilter.html"},{"type":"module","title":"Credit - Talan.BloomFilter","doc":"Partly inspired by [Blex](https://github.com/gyson/blex)","ref":"Talan.BloomFilter.html#module-credit"},{"type":"module","title":"Features - Talan.BloomFilter","doc":"* Fixed size Bloom filter\n  * Concurrent reads & writes\n  * Custom & default hash functions\n  * Merge multiple Bloom filters into one\n  * Intersect multiple Bloom filters into one\n  * Estimate number of unique elements\n  * Estimate current false positive probability","ref":"Talan.BloomFilter.html#module-features"},{"type":"module","title":"Examples - Talan.BloomFilter","doc":"iex> b = Talan.BloomFilter.new(1000)\n    iex> b |> Talan.BloomFilter.put(\"Barna\")\n    iex> b |> Talan.BloomFilter.member?(\"Barna\")\n    true\n    iex> b |> Talan.BloomFilter.member?(\"Kovacs\")\n    false","ref":"Talan.BloomFilter.html#module-examples"},{"type":"function","title":"Talan.BloomFilter.bits_info/1","doc":"Returns a map representing the bit state of the `atomics_ref`.\n\nUse this for debugging purposes.","ref":"Talan.BloomFilter.html#bits_info/1"},{"type":"function","title":"Examples - Talan.BloomFilter.bits_info/1","doc":"iex> b = Talan.BloomFilter.new(1000)\n    iex> b |> Talan.BloomFilter.bits_info()\n    %{total_bits: 9536, set_bits_count: 0, set_ratio: 0.0}","ref":"Talan.BloomFilter.html#bits_info/1-examples"},{"type":"function","title":"Talan.BloomFilter.cardinality/1","doc":"Returns an non negative integer representing the\nestimated cardinality count of unique elements in the filter.","ref":"Talan.BloomFilter.html#cardinality/1"},{"type":"function","title":"Examples - Talan.BloomFilter.cardinality/1","doc":"iex> b = Talan.BloomFilter.new(1000)\n    iex> b |> Talan.BloomFilter.cardinality()\n    0\n    iex> b |> Talan.BloomFilter.put(\"Barna\")\n    iex> b |> Talan.BloomFilter.cardinality()\n    1\n    iex> b |> Talan.BloomFilter.put(\"Barna\")\n    iex> b |> Talan.BloomFilter.cardinality()\n    1\n    iex> b |> Talan.BloomFilter.put(\"Kovacs\")\n    iex> b |> Talan.BloomFilter.cardinality()\n    2","ref":"Talan.BloomFilter.html#cardinality/1-examples"},{"type":"function","title":"Talan.BloomFilter.deserialize/1","doc":"Deserializes a binary into a Bloom filter.\n\nThis function takes a binary that was previously created by `serialize/1`\nand reconstructs the Bloom filter structure.","ref":"Talan.BloomFilter.html#deserialize/1"},{"type":"function","title":"Examples - Talan.BloomFilter.deserialize/1","doc":"iex> bloom_filter = Talan.BloomFilter.new(1000)\n    iex> serialized = Talan.BloomFilter.serialize(bloom_filter)\n    iex> deserialized = Talan.BloomFilter.deserialize(serialized)\n    iex> is_struct(deserialized, Talan.BloomFilter)\n    true","ref":"Talan.BloomFilter.html#deserialize/1-examples"},{"type":"function","title":"Talan.BloomFilter.false_positive_probability/1","doc":"Returns a float representing current estimated\nfalse positive probability.","ref":"Talan.BloomFilter.html#false_positive_probability/1"},{"type":"function","title":"Examples - Talan.BloomFilter.false_positive_probability/1","doc":"iex> b = Talan.BloomFilter.new(1000)\n    iex> b |> Talan.BloomFilter.false_positive_probability()\n    0.0 # fpp zero when bloom filter is empty\n    iex> b |> Talan.BloomFilter.put(\"Barna\") # fpp increases\n    iex> b |> Talan.BloomFilter.put(\"Kovacs\")\n    iex> fpp = b |> Talan.BloomFilter.false_positive_probability()\n    iex> fpp > 0 && fpp < 1\n    true","ref":"Talan.BloomFilter.html#false_positive_probability/1-examples"},{"type":"function","title":"Talan.BloomFilter.hash_term/2","doc":"Hashes `term` with all `hash_functions` of `%Talan.BloomFilter{}`.\n\nReturns a list of hashed values.","ref":"Talan.BloomFilter.html#hash_term/2"},{"type":"function","title":"Examples - Talan.BloomFilter.hash_term/2","doc":"b = Talan.BloomFilter.new(1000)\n    Talan.BloomFilter.hash_term(b, :any_term_can_be_hashed)\n    [9386, 8954, 8645, 4068, 5445, 6914, 2844]","ref":"Talan.BloomFilter.html#hash_term/2-examples"},{"type":"function","title":"Talan.BloomFilter.intersection/1","doc":"Intersection of `%Talan.BloomFilter{}` structs's atomics into one new struct.\n\nNote: To work correctly filters with identical size & hash functions must be used.\n\nReturns a new `%BloomFilter{}` struct which set bits are the intersection\nthe bloom filters in the `list`.","ref":"Talan.BloomFilter.html#intersection/1"},{"type":"function","title":"Examples - Talan.BloomFilter.intersection/1","doc":"iex> hash_functions = Talan.seed_n_murmur_hash_fun(7)\n    iex> b1 = Talan.BloomFilter.new(1000, hash_functions: hash_functions)\n    iex> b1 |> Talan.BloomFilter.put(\"GitHub\")\n    iex> b2 = Talan.BloomFilter.new(1000, hash_functions: hash_functions)\n    iex> b2 |> Talan.BloomFilter.put(\"GitHub\")\n    iex> b2 |> Talan.BloomFilter.put(\"Octocat\")\n    :ok\n    iex> b3 = Talan.BloomFilter.intersection([b1, b2])\n    iex> b3 |> Talan.BloomFilter.member?(\"GitHub\")\n    true\n    iex> b3 |> Talan.BloomFilter.member?(\"Octocat\")\n    false","ref":"Talan.BloomFilter.html#intersection/1-examples"},{"type":"function","title":"Talan.BloomFilter.member?/2","doc":"Checks for membership of `term` in `bloom_filter`.\n\nReturns `false` if not a member. (definitely not member)\nReturns `true` if maybe a member. (possibly member)","ref":"Talan.BloomFilter.html#member?/2"},{"type":"function","title":"Examples - Talan.BloomFilter.member?/2","doc":"iex> b = Talan.BloomFilter.new(1000)\n    iex> b |> Talan.BloomFilter.member?(\"Barna Kovacs\")\n    false\n    iex> b |> Talan.BloomFilter.put(\"Barna Kovacs\")\n    iex> b |> Talan.BloomFilter.member?(\"Barna Kovacs\")\n    true","ref":"Talan.BloomFilter.html#member?/2-examples"},{"type":"function","title":"Talan.BloomFilter.merge/1","doc":"Merge multiple `%Talan.BloomFilter{}` structs's atomics into one new struct.\n\nNote: To work correctly filters with identical size & hash functions must be used.\n\nReturns a new `%Talan.BloomFilter{}` struct which set bits are the merged set bits of\nthe bloom filters in the `list`.","ref":"Talan.BloomFilter.html#merge/1"},{"type":"function","title":"Examples - Talan.BloomFilter.merge/1","doc":"iex> hash_functions = Talan.seed_n_murmur_hash_fun(7)\n    iex> b1 = Talan.BloomFilter.new(1000, hash_functions: hash_functions)\n    iex> b1 |> Talan.BloomFilter.put(\"GitHub\")\n    iex> b2 = Talan.BloomFilter.new(1000, hash_functions: hash_functions)\n    iex> b2 |> Talan.BloomFilter.put(\"Octocat\")\n    :ok\n    iex> b3 = Talan.BloomFilter.merge([b1, b2])\n    iex> b3 |> Talan.BloomFilter.member?(\"GitHub\")\n    true\n    iex> b3 |> Talan.BloomFilter.member?(\"Octocat\")\n    true","ref":"Talan.BloomFilter.html#merge/1-examples"},{"type":"function","title":"Talan.BloomFilter.new/2","doc":"Returns a new `%Talan.BloomFilter{}` for the desired `cardinality`.\n\n`cardinality` is the expected number of unique items. Duplicated items\ncan be infinite.","ref":"Talan.BloomFilter.html#new/2"},{"type":"function","title":"Options - Talan.BloomFilter.new/2","doc":"* `:false_positive_probability` - a float, defaults to 0.01\n  * `:hash_functions` - a list of hash functions, defaults to randomly seeded murmur","ref":"Talan.BloomFilter.html#new/2-options"},{"type":"function","title":"Examples - Talan.BloomFilter.new/2","doc":"iex> bloom_filter = Talan.BloomFilter.new(1_000_000)\n    iex> bloom_filter |> Talan.BloomFilter.put(\"Barna Kovacs\")\n    :ok","ref":"Talan.BloomFilter.html#new/2-examples"},{"type":"function","title":"Talan.BloomFilter.put/2","doc":"Puts `term` into `bloom_filter` a `%Talan.BloomFilter{}` struct.\n\nAfter this the `member?` function will always return `true`\nfor the membership of `term`.\n\nReturns `:ok`.","ref":"Talan.BloomFilter.html#put/2"},{"type":"function","title":"Examples - Talan.BloomFilter.put/2","doc":"iex> b = Talan.BloomFilter.new(1000)\n    iex> b |> Talan.BloomFilter.put(\"Chris McCord\")\n    :ok\n    iex> b |> Talan.BloomFilter.put(\"Jose Valim\")\n    :ok","ref":"Talan.BloomFilter.html#put/2-examples"},{"type":"function","title":"Talan.BloomFilter.required_filter_length/2","doc":"Returns the required bit count given\n\n* `cardinality` - Number of unique elements that will be inserted\n* `false_positive_probability` - Desired false positive probability of membership\n\n[Wikipedia - Bloom filter - Optimal number of hash functions](https://en.wikipedia.org/wiki/Bloom_filter#Optimal_number_of_hash_functions)","ref":"Talan.BloomFilter.html#required_filter_length/2"},{"type":"function","title":"Examples - Talan.BloomFilter.required_filter_length/2","doc":"iex> Talan.BloomFilter.required_filter_length(10_000, 0.01)\n    95851","ref":"Talan.BloomFilter.html#required_filter_length/2-examples"},{"type":"function","title":"Talan.BloomFilter.required_hash_function_count/1","doc":"Returns count of required hash functions for the\ngiven `false_positive_probability`.\n\n[Wikipedia - Bloom filter - Optimal number of hash functions](https://en.wikipedia.org/wiki/Bloom_filter#Optimal_number_of_hash_functions)","ref":"Talan.BloomFilter.html#required_hash_function_count/1"},{"type":"function","title":"Examples - Talan.BloomFilter.required_hash_function_count/1","doc":"iex> Talan.BloomFilter.required_hash_function_count(0.01)\n    7\n    iex> Talan.BloomFilter.required_hash_function_count(0.001)\n    10\n    iex> Talan.BloomFilter.required_hash_function_count(0.0001)\n    14","ref":"Talan.BloomFilter.html#required_hash_function_count/1-examples"},{"type":"function","title":"Talan.BloomFilter.serialize/1","doc":"Serializes the Bloom filter into a binary.\n\nThis function converts the Bloom filter structure into a binary format,\nwhich can be used for storage or transmission.","ref":"Talan.BloomFilter.html#serialize/1"},{"type":"function","title":"Examples - Talan.BloomFilter.serialize/1","doc":"iex> bloom_filter = Talan.BloomFilter.new(1000)\n    iex> serialized = Talan.BloomFilter.serialize(bloom_filter)\n    iex> is_binary(serialized)\n    true","ref":"Talan.BloomFilter.html#serialize/1-examples"},{"type":"type","title":"Talan.BloomFilter.t/0","doc":"","ref":"Talan.BloomFilter.html#t:t/0"},{"type":"module","title":"Talan.Counter","doc":"Linear probabilistic counter implementation with **concurrent accessibility**,\npowered by [:atomics](http://erlang.org/doc/man/atomics.html) module for cardinality estimation.\n\nCardinality is the count of unique elements.\n\nFor more info about linear probabilistic counting:\n[linear probabilistic counting](https://www.waitingforcode.com/big-data-algorithms/cardinality-estimation-linear-probabilistic-counting/read)","ref":"Talan.Counter.html"},{"type":"function","title":"Talan.Counter.cardinality/1","doc":"Returns the estimated cardinality for the given\n`%Talan.Counter{}` struct.","ref":"Talan.Counter.html#cardinality/1"},{"type":"function","title":"Examples - Talan.Counter.cardinality/1","doc":"iex> c = Talan.Counter.new(10_000)\n    iex> c |> Talan.Counter.put([\"you\", :can, Hash, {\"any\", \"elixir\", \"term\"}])\n    iex> c |> Talan.Counter.put([\"you\", :can, Hash, {\"any\", \"elixir\", \"term\"}])\n    iex> c |> Talan.Counter.cardinality()\n    1\n    iex> c |> Talan.Counter.put(\"more\")\n    iex> c |> Talan.Counter.cardinality()\n    2","ref":"Talan.Counter.html#cardinality/1-examples"},{"type":"function","title":"Talan.Counter.new/2","doc":"Returns a new `%Talan.Counter{}` struct.\n\n`expected_cardinality` is the max number of uniq items the counter will\nhandle with approx 1% of error rate.","ref":"Talan.Counter.html#new/2"},{"type":"function","title":"Options - Talan.Counter.new/2","doc":"* `hash_function` - defaults to `Murmur.hash_x64_128/1`","ref":"Talan.Counter.html#new/2-options"},{"type":"function","title":"Examples - Talan.Counter.new/2","doc":"iex> c = Talan.Counter.new(10_000)\n    iex> c |> Talan.Counter.put([\"you\", :can, Hash, {\"any\", \"elixir\", \"term\"}])\n    iex> c |> Talan.Counter.put(\"more\")\n    iex> c |> Talan.Counter.put(\"another\")\n    iex> c |> Talan.Counter.cardinality()\n    3","ref":"Talan.Counter.html#new/2-examples"},{"type":"function","title":"Talan.Counter.put/2","doc":"Hashes `term` and sets a bit to mark it has been seen.\n\nDoesn't store the `term` so it's space efficient.\nUses `:atomics` so it's mutable & highly concurrent.\n\nReturns `:ok`.","ref":"Talan.Counter.html#put/2"},{"type":"function","title":"Examples - Talan.Counter.put/2","doc":"iex> c = Talan.Counter.new(10_000)\n    iex> c |> Talan.Counter.put([\"you\", :can, Hash, {\"any\", \"elixir\", \"term\"}])\n    :ok","ref":"Talan.Counter.html#put/2-examples"},{"type":"type","title":"Talan.Counter.t/0","doc":"","ref":"Talan.Counter.html#t:t/0"},{"type":"module","title":"Talan.CountingBloomFilter","doc":"Counting bloom filter implementation with **concurrent accessibility**,\npowered by [:atomics](http://erlang.org/doc/man/atomics.html) module.","ref":"Talan.CountingBloomFilter.html"},{"type":"module","title":"Features - Talan.CountingBloomFilter","doc":"* Fixed size Counting Bloom filter\n  * Concurrent reads & writes\n  * Custom & default hash functions\n  * Estimate number of unique elements\n  * Estimate false positive probability\n\nCounting bloom filters support probabilistic deletion\nof elements but have higher memory consumption because\nthey need to store a counter of N bits for every bloom filter bit.","ref":"Talan.CountingBloomFilter.html#module-features"},{"type":"function","title":"Talan.CountingBloomFilter.cardinality/1","doc":"See `Talan.BloomFilter.cardinality/1` for docs.","ref":"Talan.CountingBloomFilter.html#cardinality/1"},{"type":"function","title":"Examples - Talan.CountingBloomFilter.cardinality/1","doc":"iex> cbf = Talan.CountingBloomFilter.new(10_000)\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    iex> cbf |> Talan.CountingBloomFilter.put(\"car keys\")\n    iex> cbf |> Talan.CountingBloomFilter.cardinality()\n    2","ref":"Talan.CountingBloomFilter.html#cardinality/1-examples"},{"type":"function","title":"Talan.CountingBloomFilter.count/2","doc":"Returns probabilistic count of term in `counter`.\n\nThis means that (given no hash collisions) it returns how many times\nthe item was put into the CountingBloomFilter. A few hash collisions\nshould be also fine since it returns the average count of the counters.\nAn single item is hashed with multiple counters.","ref":"Talan.CountingBloomFilter.html#count/2"},{"type":"function","title":"Examples - Talan.CountingBloomFilter.count/2","doc":"iex> cbf = Talan.CountingBloomFilter.new(10_000)\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    iex> cbf |> Talan.CountingBloomFilter.count(\"hat\")\n    3","ref":"Talan.CountingBloomFilter.html#count/2-examples"},{"type":"function","title":"Talan.CountingBloomFilter.delete/2","doc":"Probabilistically delete `term` from `bloom_filter` and\ndecrement counters in `counter`.","ref":"Talan.CountingBloomFilter.html#delete/2"},{"type":"function","title":"Examples - Talan.CountingBloomFilter.delete/2","doc":"iex> cbf = Talan.CountingBloomFilter.new(10_000)\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    iex> cbf |> Talan.CountingBloomFilter.count(\"hat\")\n    1\n    iex> cbf |> Talan.CountingBloomFilter.delete(\"hat\")\n    :ok\n    iex> cbf |> Talan.CountingBloomFilter.count(\"hat\")\n    0\n    iex> cbf |> Talan.CountingBloomFilter.delete(\"this wasn't there\")\n    iex> cbf |> Talan.CountingBloomFilter.count(\"this wasn't there\")\n    -1","ref":"Talan.CountingBloomFilter.html#delete/2-examples"},{"type":"function","title":"Talan.CountingBloomFilter.false_positive_probability/1","doc":"See `Talan.BloomFilter.false_positive_probability/1` for\ndocs.","ref":"Talan.CountingBloomFilter.html#false_positive_probability/1"},{"type":"function","title":"Talan.CountingBloomFilter.member?/2","doc":"See `Talan.BloomFilter.member?/2` for docs.","ref":"Talan.CountingBloomFilter.html#member?/2"},{"type":"function","title":"Examples - Talan.CountingBloomFilter.member?/2","doc":"iex> cbf = Talan.CountingBloomFilter.new(10_000)\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    iex> cbf |> Talan.CountingBloomFilter.member?(\"hat\")\n    true","ref":"Talan.CountingBloomFilter.html#member?/2-examples"},{"type":"function","title":"Talan.CountingBloomFilter.new/2","doc":"Returns a new `%Talan.CountingBloomFilter{}` struct.\n\n`cardinality` is the expected number of unique items. Duplicated items\ncan be infinite.","ref":"Talan.CountingBloomFilter.html#new/2"},{"type":"function","title":"Options - Talan.CountingBloomFilter.new/2","doc":"* `:counters_bit_size` - bit size of counters, defaults to `8`\n  * `:signed` - to have signed or unsigned counters, defaults to `true`\n  * `:false_positive_probability` - a float, defaults to `0.01`\n  * `:hash_functions` - a list of hash functions, defaults to randomly seeded murmur","ref":"Talan.CountingBloomFilter.html#new/2-options"},{"type":"function","title":"Examples - Talan.CountingBloomFilter.new/2","doc":"iex> cbf = Talan.CountingBloomFilter.new(10_000)\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    iex> cbf |> Talan.CountingBloomFilter.put(\"phone\")\n    :ok\n    iex> cbf |> Talan.CountingBloomFilter.count(\"hat\")\n    2\n    iex> cbf |> Talan.CountingBloomFilter.count(\"phone\")\n    1","ref":"Talan.CountingBloomFilter.html#new/2-examples"},{"type":"function","title":"Talan.CountingBloomFilter.put/2","doc":"Puts `term` into `bloom_filter` and increments counters in `counter`.\n\nAfter this the `member?/2` function will return `true`\nfor the membership of `term` unless bits representing\nmembership are modified by the `delete/2` function.\n\nReturns `:ok`.","ref":"Talan.CountingBloomFilter.html#put/2"},{"type":"function","title":"Examples - Talan.CountingBloomFilter.put/2","doc":"iex> cbf = Talan.CountingBloomFilter.new(10_000)\n    iex> cbf |> Talan.CountingBloomFilter.put(\"hat\")\n    :ok","ref":"Talan.CountingBloomFilter.html#put/2-examples"},{"type":"type","title":"Talan.CountingBloomFilter.t/0","doc":"","ref":"Talan.CountingBloomFilter.html#t:t/0"},{"type":"module","title":"Talan.Stream","doc":"","ref":"Talan.Stream.html"},{"type":"function","title":"Talan.Stream.uniq/2","doc":"Returns a probabilistically uniq stream.\n\nIts main advantage is that it doesn't store elements\nemitted by the stream.\nInstead it uses a bloom filter for membership check.\n\nThe stream never returns duplicate elements but it\nsometimes detects false positive duplicates depending\non the bloom filter it uses.\nFalse positives are faulty duplicate detections that\nget rejected.","ref":"Talan.Stream.html#uniq/2"},{"type":"function","title":"Examples - Talan.Stream.uniq/2","doc":"iex> list = [\"a\", \"b\", \"c\", \"a\", \"b\"]\n    iex> bloom_filter = Talan.BloomFilter.new(100_000, false_positive_probability: 0.001)\n    iex> Talan.Stream.uniq(list, bloom_filter) |> Enum.to_list\n    [\"a\", \"b\", \"c\"]","ref":"Talan.Stream.html#uniq/2-examples"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}